#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages

getmethod = robjects.baseenv.get("getMethod")


__rname__ = 'AnnotationDbi'
rpy2.robjects.packages.quiet_require(__rname__)

package_env = robjects.baseenv['as.environment']('package:AnnotationDbi')
__rpackage__ = robjects.packages.SignatureTranslatedPackage(package_env, __rname__)

StrVector = robjects.StrVector

class Bimap(rpy2.robjects.methods.RS4):
    ''' A Bimap as defined in the R package AnnotationDbi '''

    _length = getmethod("length", 
                        signature = StrVector(["Bimap", ]),
                        where="package:AnnotationDbi")
    _keys = getmethod("keys", 
                      signature = StrVector(["Bimap", ]),
                      where="package:AnnotationDbi")
    _mappedkeys = getmethod("mappedkeys", 
                            signature = StrVector(["Bimap", ]),
                            where="package:AnnotationDbi")
    _count_mappedkeys = getmethod("count.mappedkeys", 
                                  signature = StrVector(["Bimap", ]),
                                  where="package:AnnotationDbi")
    _subset = getmethod("subset", 
                        signature = StrVector(["Bimap", ]),
                        where="package:AnnotationDbi")


    def __length__(self):
        res = self._length(self)[0]
        return res

    def keys(self):
        res = self._keys(self)
        return res

    def mappedkeys(self):
        res = self._mappedkeys(self)
        return res

    def count_mappedkeys(self):
        res = self._count_mappedkeys(self)
        return res

    def subset(self, lkeys = None, rkeys = None):
        res = self._subset(self, Lkeys = lkeys, Rkeys = rkeys)
        return res


class AnnDbBimap(Bimap):
    _lkeys_get = getmethod("Lkeys", 
                       signature = StrVector(["AnnDbBimap", ]),
                       where="package:AnnotationDbi")
    _lkeys_set = getmethod("Lkeys<-", 
                           signature = StrVector(["AnnDbBimap", ]),
                           where="package:AnnotationDbi")
    _mappedlkeys = getmethod("mappedLkeys", 
                             signature = StrVector(["AnnDbBimap", ]),
                             where="package:AnnotationDbi")
    _rkeys_get = getmethod("Rkeys", 
                           signature = StrVector(["AnnDbBimap", ]),
                           where="package:AnnotationDbi")
    _rkeys_set = getmethod("Rkeys<-", 
                           signature = StrVector(["AnnDbBimap", ]),
                           where="package:AnnotationDbi")
    _mappedrkeys = getmethod("mappedRkeys", 
                             signature = StrVector(["AnnDbBimap", ]),
                             where="package:AnnotationDbi")
    _subset = getmethod("subset", 
                        signature = StrVector(["AnnDbBimap", ]),
                        where="package:AnnotationDbi")

    def get_lkeys(self):
        res = self._lkeys_get(self)
        return res
    def set_lkeys(self, value):
        sexp = self._lkeys_set(self, value)
        self.__sexp__ = sexp
    lkeys = property(get_lkeys, set_lkeys, None)

    def mappedlkeys(self):
        res = self._mappedlkeys(self)
        return res

    def get_rkeys(self):
        res = self._rkeys_get(self)
        return res
    def set_rkeys(self, value):
        sexp = self._rkeys_set(self, value)
        self.__sexp__ = sexp
    rkeys = property(get_rkeys, set_rkeys, None)


    def mappedrkeys(self):
        res = self._mappedrkeys(self)
        return res

    def subset(self, lkeys = None, rkeys = None, objname = None):
        res = self._subset(self, Lkeys = lkeys, Rkeys = rkeys, 
                           objName = objname)
        return res


class AnnDbMap(AnnDbBimap):
    pass

class GOTerms(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('GOID', 'AnnotationDbi', 'GOID', True,
                      'maps AnnotationDbi::GOID'),
                     ('Term', 'AnnotationDbi', 'term', True,
                      'maps AnnotationDbi::Term'),
                     ('Ontology', 'AnnotationDbi', 'ontology', True,
                      'maps AnnotationDbi::Ontology'),
                     ('Definition', 'AnnotationDbi', 'definition', True,
                      'maps AnnotationDbi::Definition'),
                     ('Synonym', 'AnnotationDbi', 'synonym', True,
                      'maps AnnotationDbi::Synonym'),
                     ('Secondary', 'AnnotationDbi', 'secondary', True,
                      'maps AnnotationDbi::Secondary'),
                     )


annotationdbi_dict = {
    'Bimap': Bimap,
    'AnnDbBimap': AnnDbBimap,
    'AnnDbMap': AnnDbMap
    }

original_conversion = robjects.conversion.ri2py
def annotationdbi_conversion(robj):
    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = annotationdbi_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

robjects.conversion.ri2py = annotationdbi_conversion




#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the Biostrings library in Bioconductor

Copyright 2009-2010 - Laurent Gautier

"""


import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages
conversion = robjects.conversion
import iranges
from rpy2.robjects.packages import importr

base = importr('base')
methods = importr('methods')
getmethod = methods.getMethod
__rname__ = 'Biostrings'

biostrings = importr(__rname__)
__rpackage__ = biostrings

StrVector = robjects.StrVector



def _setExtractDelegators(self):
    self.rx = robjects.vectors.ExtractDelegator(self)
    self.rx2 = robjects.vectors.DoubleExtractDelegator(self)        

   
class XString(rpy2.robjects.methods.RS4):
    """ Arbitrary string """

    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('nchar', 'Biostrings', None, True, None),
                     ('reverse', 'Biostrings', None, False, None))

    _alphabet = getmethod("alphabet", 
                          signature = StrVector(["ANY", ]), 
                          where="package:Biostrings")


    def __init__(self, *args, **kwargs):
        super(XString, self).__init__(*args, **kwargs)
        _setExtractDelegators(self)
        
    def get_alphabet(self):
        res = self._alphabet(self)
        return res
    alphabet = property(get_alphabet, None, "Get the 'alphabet'")




class BString(XString):
    """ Biological string """
    _bstring_constructor = biostrings.BString

    @classmethod
    def new(cls, x):
        """ :param x: a (biological) string """
        res = cls(cls._bstring_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res


class DNAString(XString):
    ''' DNA string '''
    _dnastring_constructor = biostrings.DNAString
    _reverse_complement = getmethod("reverseComplement", 
                                    signature = StrVector(["DNAString", ]), 
                                    where="package:Biostrings")

    @classmethod
    def new(cls, x):
        """ :param x: a DNA string """
        res = cls(cls._dnastring_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res

    def reverse_complement(self):
        ''' Return the reverse complement '''
        res = self._reverse_complement(self)
        return res


    

class RNAString(XString):
    """ RNA string """
    _rnastring_constructor = biostrings.RNAString
    _reverse_complement = getmethod("reverseComplement", 
                                    signature = StrVector(["RNAString", ]), 
                                    where="package:Biostrings")

    @classmethod
    def new(cls, x):
        """ :param x: an RNA string """
        res = cls(cls._rnastring_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res

    def reverse_complement(self):
        ''' Return the reverse complement '''
        res = self._reverse_complement(self)
        return res


class AAString(XString):
    _aastring_constructor = biostrings.AAString

    @classmethod
    def new(cls, x):
        """ :param x: a string of amino-acids """
        res = cls(cls._aastring_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res


class XStringSet(rpy2.robjects.methods.RS4):
    """ An abstract class, parent to *StringSet classes """
    _subset = getmethod("[", 
                        signature = "XStringSet",
                        where = "package:Biostrings")
    _length = base.length

    r = None
    
    def subset(self, x):
        res = self._subset(self, x)
        return res

    #FIXME: should implement an rpy2-defined sequence interface ?
    def __len__(self):
        res = self._length(self)[0]
        return res

class BStringSet(XStringSet):
    ''' Set of biological strings '''
    _bstringset_constructor = biostrings.BStringSet
    @classmethod
    def new(cls, x):
        res = cls(cls._bstringset_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res


class DNAStringSet(XStringSet):
    ''' Set of DNA strings '''
    _dnastringset_constructor = biostrings.DNAStringSet

    @classmethod
    def new(cls, x):
        res = cls(cls._dnastringset_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res


class RNAStringSet(XStringSet):
    ''' Set of RNA strings '''
    _rnastringset_constructor = biostrings.RNAStringSet
    @classmethod
    def new(cls, x):
        res = cls(cls._rnastringset_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res


class AAStringSet(XStringSet):
    ''' Set of amino-acid strings '''
    _aastringset_constructor = biostrings.AAStringSet
    @classmethod
    def new(cls, x):
        res = cls(cls._aastringset_constructor(conversion.py2ri(x)))
        _setExtractDelegators(res)
        return res

class AlignedXStringSet0(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    __accessors__ = (('unaligned', 'Biostrings', None, True, None),
                     ('start', 'Biostrings', None, True, None),
                     ('end', 'Biostrings', None, True, None),
                     ('width', 'Biostrings', None, True, None),
                     ('indel', 'Biostrings', None, False, None),
                     ('length', 'Biostrings', '__len__', False, None),
                     ('nchar', 'Biostrings', None, True, None),
                     #('alphabet', None, True, ''),
                     ('as.character', 'Biostrings', 'as_character',
                      False, None))

    _alphabet = getmethod("alphabet", 
                          signature = StrVector(["ANY", ]), 
                          where="package:Biostrings")
    def get_alphabet(self):
        res = self._alphabet(self)
        return res
    alphabet = property(get_alphabet, None, "Get the 'alphabet'")


class AlignedXStringSet(AlignedXStringSet0):
    pass


class XStringViews(iranges.Views):
    """ View on an arbitrary string """

    
    _nchar = getmethod("nchar", 
                       signature = "XStringViews",
                       where="package:Biostrings")
    _reverse_complement = getmethod("reverseComplement", 
                                    signature = StrVector(["XStringViews", ]), 
                                    where="package:Biostrings")
    _as_matrix = getmethod("as.matrix", 
                           signature = StrVector(["XStringViews", ]), 
                           where="package:Biostrings")

        

    def get_nchar(self):
        res = self._nchar(self)
        return res
    nchar = property(get_nchar, None,
                     "Number of characters in the view")
    width = property(get_nchar, None,
                     "Number of characters in the view (identical to nchar)")

    
    def reverse_complement(self):
        """ Return the reverse and complement for the view """
        res = self._reverse_complement(self)
        return res

    def as_matrix(self, **kwargs):
        res = self._as_matrix(self, **kwargs)
        return res

class MaskedXString(rpy2.robjects.methods.RS4):
    ''' "Masked" arbitrary string '''
    _unmasked = getmethod("unmasked", 
                          signature = "MaskedXString",
                          where="package:Biostrings")
    _masks = getmethod("masks", 
                       signature = "MaskedXString",
                       where="package:Biostrings")
    _masks_set_maskcollection = getmethod("masks<-", 
                                          signature = robjects.StrVector(("MaskedXString", 
                                                                          "MaskCollection")),
                                          where="package:Biostrings")
    

    def __length__(self):
        res = self._length(self)
        return res

    def get_unmasked(self):
        """ Return the strings without its 'mask' """
        res = self._unmasked(self)
        return res
    unmasked = property(get_unmasked, None)

    def get_masks(self):
        res = res._masks(self)
        return res

    def set_masks(self, value):
        if isinstance(value, MaskCollection):
            res = res._masks_set_maskcollection(self, value)
        else:
            raise ValueError('Value should be a MaskCollection')
        return res
    masks = property(get_masks, set_masks,
                     None,
                     "Property for both R's 'masks' and 'masks<-'")


class MaskedBString(MaskedXString):
    """ Masked biological string """
    pass

class MaskedDNAString(MaskedXString):
    """ Masked DNA string """
    pass

class MaskedRNAString(MaskedXString):
    """ Masked RNA string """
    pass

class MaskedAAString(MaskedXString):
    """ Masked string of amino-acids """
    pass

class XStringQuality(rpy2.robjects.methods.RS4):
    pass

class PhredQuality(XStringQuality, XStringSet):
    _constructor = biostrings.PhredQuality

    @classmethod
    def new(cls, x):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res

    
class SolexaQuality(XStringQuality, XStringSet):
    _constructor = biostrings.SolexaQuality

    @classmethod
    def new(cls, x):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res

class QualityScaledDNAStringSet(XStringSet):
    _constructor = biostrings.QualityScaledDNAStringSet
    @classmethod
    def new(cls, x, quality):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res


class QualityScaledXStringSet(XStringSet):
    pass

# acs = (
#        ('quality', None, True, None),
#       )
# rpy2.robjects.methods.set_accessors(QualityScaledXStringSet,
#                                     "QualityScaledXStringSet", "Biostrings",
#                                     acs)
# del(acs)

class QualityScaledDNAStringSet(DNAStringSet, QualityScaledXStringSet):
    _constructor = biostrings.QualityScaledDNAStringSet
    @classmethod
    def new(cls, x, quality):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res

class QualityScaledRNAStringSet(RNAStringSet, QualityScaledXStringSet):
    _constructor = biostrings.QualityScaledRNAStringSet
    @classmethod
    def new(cls, x, quality):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res

class QualityScaledBStringSet(BStringSet, QualityScaledXStringSet):
    _constructor = biostrings.QualityScaledBStringSet
    @classmethod
    def new(cls, x, quality):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res

class QualityScaledAAStringSet(AAStringSet, QualityScaledXStringSet):
    _constructor = biostrings.QualityScaledAAStringSet
    @classmethod
    def new(cls, x, quality):
        res = cls(cls._constructor(conversion.py2ri(x)))
        return res


    
    
class InDel(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    __accessors__ = (('insertion', 'Biostrings', None, True, None),
                     ('deletion', 'Biostrings', None, True, None))


class PairwiseAlignedXStringSet(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    __accessors__ = (('score', 'Biostrings', None, True, None),
                     ('nchar', 'Biostrings', None, True, None),
                     ('nindel', 'Biostrings', None, True, None),
                     ('pattern', 'Biostrings', None, True, None),
                     )
        

    _fromXString_XString = \
            getmethod("PairwiseAlignedXStringSet", 
                      signature = robjects.StrVector(("XString",
                                                      "XString")),
                      where="package:Biostrings")
    _fromCharacter_Character = \
            getmethod("PairwiseAlignedXStringSet", 
                      signature = robjects.StrVector(("character",
                                                      "character")),
                                       where="package:Biostrings")

    _fromXStringSet_missing = \
            getmethod("PairwiseAlignedXStringSet", 
                      signature = robjects.StrVector(("XStringSet",
                                                      "missing")),
                                   where="package:Biostrings")
    _fromCharacter_missing = \
            getmethod("PairwiseAlignedXStringSet", 
                      signature = robjects.StrVector(("character",
                                                      "missing")),
                      where="package:Biostrings")
    

    @staticmethod
    def fromXString_XString(pattern, target, **kwargs):
        res = PairwiseAlignedXStringSet._fromXString_XString(pattern,
                                                             target)
        return res

    @staticmethod
    def fromCharacter_Character(pattern, target, **kwargs):
        res = PairwiseAlignedXStringSet._fromCharacter_Character(pattern,
                                                                 target,
                                                                 **kwargs)
        return res

    @staticmethod
    def fromCharacter_missing(pattern, **kwargs):
        res = PairwiseAlignedXStringSet._fromCharacter_missing(pattern,
                                                               **kwargs)
        return res


class PairwiseAlignedFixedSubject(PairwiseAlignedXStringSet):
    pass

class PairwiseAlignedFixedSubjectSummary(rpy2.robjects.methods.RS4):
    pass

class PDict(rpy2.robjects.methods.RS4):
    """ Dictionnary of probes, that is dictionary of
    of rather short strings. """
    
    _length = getmethod("length", 
                        signature = "PDict",
                        where = "package:Biostrings")
    _pdict_constructor = biostrings.PDict
    _match = biostrings._env['matchPDict']
    _count = biostrings._env['countPDict']
    _which = biostrings._env['whichPDict']
    _width = getmethod("width",
                       signature = "PDict",
                       where = "package:Biostrings")

    @classmethod
    def create_instance(self, x):
        """ Create a preprocessed dictionnary of genomic patterns.

        :param x: a string vector, and DNAStringSet, 
                  or an XStringViews with s DNAString subject"""
        res = self._pdict_constructor(conversion.py2ri(x))
        return res
        
    def __length__(self):
        res = self._length(self)
        return res

    width = property(__length__, None,
                     "Number of elements in the dictionary (identical to length)")

    
    def match(self, subject, algorithm = "auto", 
              max_mismatch = 0, fixed = True, verbose = False):
        """ Match subject sequence(s) to the dictionary """
        res = self._match(self, subject, 
                          **{'algorithm': algorithm, 
                             'max.mismatch': max_mismatch,
                             'fixed': fixed, 
                             'verbose': verbose})
        return res

    def count(self, subject, algorithm = "auto", 
              max_mismatch = 0, fixed = True, verbose = False):
        """ Count the number of matching subject sequences """
        res = self._match(self, subject, 
                          **{'algorithm': algorithm, 
                             'max.mismatch': max_mismatch,
                             'fixed': fixed, 
                             'verbose': verbose})
        return res
    def which(self, subject, algorithm = "auto", 
              max_mismatch = 0, fixed = True, verbose = False):
        res = self._match(self, subject, 
                          **{'algorithm': algorithm, 
                             'max.mismatch': max_mismatch,
                             'fixed': fixed, 
                             'verbose': verbose})
        return res

class TB_PDict(PDict):
    """ 'Trusted-band' (TB) probe dictionary """
    _tb = getmethod("tb", 
                    signature = "TB_PDict",
                    where = "package:Biostrings")
    _tb_width = getmethod("tb.width", 
                          signature = "TB_PDict",
                          where = "package:Biostrings")
    def get_tb(self):
        res = self._tb(self)
        return res
    tb = property(get_tb, None,
                  "Return the trusted band for the dictionary")

    def get_tb_width(self):
        res = self._tb_width(self)
        return res
    tb_width = property(get_tb_width, None,
                        "Return the width of the trusted band")
   

class MTB_PDict(PDict):
    pass


_biostrings_dict = {
    'BString': BString, 
    'DNAString': DNAString,
    'RNAString': RNAString,
    #
    #'Views': Views,
    'XStringViews': XStringViews,
    #
    #'MaskCollection': MaskCollection,
    'MaskedXString': MaskedXString,
    'MaskedBString': MaskedBString,
    'MaskedDNAString': MaskedDNAString,
    'MaskedRNAString': MaskedRNAString,
    #
    'PDict': PDict,
    'TB_PDict': TB_PDict,
    'MTB_PDict': MTB_PDict,
    #
    'XStringSet': XStringSet,
    'BStringSet': BStringSet,
    'DNAStringSet': DNAStringSet,
    'RNAStringSet': RNAStringSet,
    'AAStringSet': AAStringSet,
    #
    'AlignedXStringSet0': AlignedXStringSet0,
    'AlignedXStringSet': AlignedXStringSet,
    #
    'XStringQuality': XStringQuality,
    'PhredQuality': PhredQuality,
    'SolexaQuality': SolexaQuality,
    #
    'QualityScaledXStringSet': QualityScaledXStringSet,
    'QualityScaledBStringSet': QualityScaledBStringSet,
    'QualityScaledDNAStringSet': QualityScaledDNAStringSet,
    'QualityScaledRNAStringSet': QualityScaledRNAStringSet,
    'QualityScaledAAStringSet': QualityScaledAAStringSet,
    #
    'InDel': InDel,
    #
    'PairwiseAlignedXStringSet': PairwiseAlignedXStringSet,
    'PairwiseAlignedFixedSubject': PairwiseAlignedFixedSubject,
    'PairwiseAlignedFixedSubjectSummary': PairwiseAlignedFixedSubjectSummary
    }

original_conversion = conversion.ri2py
def biostrings_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _biostrings_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

conversion.ri2py = biostrings_conversion




#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the GGtools library in Bioconductor

Copyright 2009 - Laurent Gautier

"""

__rname__ = 'GGBase'

import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages

conversion = robjects.conversion
getmethod = robjects.baseenv.get("getMethod")
import bioc.biobase
rpy2.robjects.packages.quiet_require(__rname__)

package_env = robjects.baseenv['as.environment']('package:%s' %__rname__)
__rpackage__ = robjects.packages.SignatureTranslatedPackage(package_env, 
                                                            __rname__)

StrVector = robjects.StrVector

class SmlSet(bioc.biobase.ESet):
    __rname__ = 'smlSet'
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('smList', 'GGBase', 'smlist', True, None),
                     ('smlEnv', 'GGBase', 'smlenv', True, None),
                     ('exprs', 'GGBase', 'exprs', True, None)
                     )
    _getalleles = getmethod("getAlleles", 
                            signature = StrVector(["smlSet", "rsid"]), 
                            where="package:GGBase")
    _snps = getmethod("snps", 
                      signature = StrVector(["smlSet", "chrnum"]), 
                      where="package:GGBase")
    def getalleles(self, rsid):
        res = self._getalleles(self, rsid)
        res = conversion.ri2py(res)
        return res
    def snps(self, chrnum):
        res = self._snps(self, chrnum)
        res = conversion.ri2py(res)
        return res


class GeneSymbol(bioc.biobase.CharacterOrMIAME):
    __rname__ = 'gensym'
    pass

_ggbase_dict = {
    'smlSet': SmlSet,
    'genesym': GeneSymbol
    }

original_conversion = robjects.conversion.ri2py
def ggbase_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _ggbase_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

robjects.conversion.ri2py = ggbase_conversion



import unittest

import rpy2.robjects as robjects
import bioc.biostrings as bs

from test_utils import RS4Test_Type


class XStringTestCase(unittest.TestCase):
    __metaclass__ = RS4Test_Type
    __targetclass__ = bs.XString
    obj = None

    def setUp(self):
        self.obj = bs.DNAString.new("ATTACG")

    def test_nchar(self):
        xs = self.obj
        nchar = xs.nchar
        self.assertEquals(1, len(nchar))
        self.assertEquals(6, nchar[0])

class DNAStringTestCase(unittest.TestCase):
    
    def test___init__(self):
        ds = bs.DNAString.new("ATTACG")
        self.assertEquals(6, ds.nchar[0])

class RNAStringTestCase(unittest.TestCase):
    
    def test___init__(self):
        ds = bs.RNAString.new("AUUACG")
        self.assertEquals(6, ds.nchar[0])

class AAStringTestCase(unittest.TestCase):
    
    def test___init__(self):
        ds = bs.AAString.new("AULVCG")
        self.assertEquals(6, ds.nchar[0])


class DNAStringSetTestCase(unittest.TestCase):
    
    def test___init__(self):
        strvect = robjects.StrVector(("ATTACG", "ATTAAT"))
        dss = bs.DNAStringSet.new(strvect)
        self.assertEquals(2, len(dss))


class AlignedXStringSet0TestCase(unittest.TestCase):
    __metaclass__ = RS4Test_Type
    __targetclass__ = bs.AlignedXStringSet0
    obj = None

    def setUp(self):
        self.obj = robjects.r('''
        local({
        pattern <- AAString("LAND")
        subject <- AAString("LEAVES")
        nw1 <- pairwiseAlignment(pattern, subject,
                                 substitutionMatrix = "BLOSUM50",
                                 gapOpening = -3, gapExtension = -1)
        pattern(nw1)
        })''')
    
    def test_nchar(self):
        xs = self.obj
        nchar = xs.nchar
        self.assertEquals(1, len(nchar))
        self.assertEquals(6, nchar[0])


class InDelTestCase(unittest.TestCase):
    __metaclass__ = RS4Test_Type
    __targetclass__ = bs.InDel
    obj = None

    def setUp(self):
        self.obj = robjects.r('''
        local({
        pattern <- AAString("LAND")
        subject <- AAString("LEAVES")
        nw1 <- pairwiseAlignment(pattern, subject,
                                 substitutionMatrix = "BLOSUM50",
                                 gapOpening = -3, gapExtension = -1)
        nindel(nw1)
        })''')
    

class PairwiseAlignedXStringSetTestCase(unittest.TestCase):
    __metaclass__ = RS4Test_Type
    __targetclass__ = bs.PairwiseAlignedXStringSet
    obj = None

    def setUp(self):
        self.obj = robjects.r('''
        local({
        pattern <- AAString("LAND")
        subject <- AAString("LEAVES")
        nw1 <- pairwiseAlignment(pattern, subject,
                                 substitutionMatrix = "BLOSUM50",
                                 gapOpening = -3, gapExtension = -1)
        nw1
        })''')
    

def suite():
    tl = unittest.TestLoader()
    suite = tl.loadTestsFromTestCase(XStringTestCase)
    suite.addTest(tl.loadTestsFromTestCase(DNAStringTestCase))
    suite.addTest(tl.loadTestsFromTestCase(RNAStringTestCase))
    suite.addTest(tl.loadTestsFromTestCase(AAStringTestCase))
    suite.addTest(tl.loadTestsFromTestCase(DNAStringSetTestCase))
    suite.addTest(tl.loadTestsFromTestCase(AlignedXStringSet0TestCase))
    suite.addTest(tl.loadTestsFromTestCase(InDelTestCase))
    suite.addTest(tl.loadTestsFromTestCase(PairwiseAlignedXStringSetTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()



import unittest

import rpy2.robjects as robjects
import bioc.bsgenome as bg
from test_utils import RS4Test_Type

ridentical = robjects.r["identical"]


class GenomeDescriptionTestCase(unittest.TestCase):
    __metaclass__ = RS4Test_Type
    __targetclass__ = bg.GenomeDescription
    obj = None
    
    def setUp(self):
        robjects.r('library(BSgenome.Celegans.UCSC.ce2)')
        self.obj = robjects.r['Celegans']



def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(GenomeDescriptionTestCase)
    #suite.addTest(unittest.TestLoader().loadTestsFromTestCase(FooTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()



#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the BSgenome library in Bioconductor

Copyright 2009 - Laurent Gautier

"""


import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages
import bioc.iranges, bioc.biostrings


getmethod = robjects.baseenv.get("getMethod")


__rname__ = 'BSgenome'
rpy2.robjects.packages.quiet_require(__rname__)

bsgenome_env = robjects.baseenv['as.environment']('package:BSgenome')
__rpackage__ = robjects.packages.SignatureTranslatedPackage(bsgenome_env, __rname__)

StrVector = robjects.StrVector


class GenomeDescription(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('organism', 'BSgenome', None, True, None),
                     ('species', 'BSgenome', None, True, None),
                     ('provider', 'BSgenome', None, True, None),
                     ('providerVersion', 'BSgenome', 'provider_version',
                      True, None),
                     ('releaseDate', 'BSgenome', 'release_date', True, None),
                     ('releaseName', 'BSgenome', 'release_name', True, None),
                     )



class BSgenome(GenomeDescription):
    """ Arbitrary string """

    __accessors__ = (('seqnames', 'BSgenome', None, True, None),
                     ('mseqnames', 'BSgenome', None, True, None),
                     ('seqlengths', 'BSgenome', None, True, None),
                     )
    
    _extract = getmethod("[[", 
                         signature = StrVector(["BSgenome", "ANY", "ANY"]),
                         where="package:BSgenome")


    def __getitem__(self, x):
        if isinstance(x, int):
            x = x - 1
        elif not (isinstance(x, str) or isinstance(x, unicode)):
            raise ValueError('x should either be an integer or a string/unicode')
        res = self._extract(self, x)
        return res

class GenomeData(bioc.iranges.AnnotatedList):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('organism', 'BSgenome', None, True, None),
                     ('provider', 'BSgenome', None, True, None),
                     ('providerVersion', 'BSgenome', 'provider_version',
                      True, None),
                     )

class GenomeDataList(bioc.iranges.AnnotatedList):
    pass

_bsgenome_dict = {
    'GenomeDescription': GenomeDescription,
    'BSgenome': BSgenome,
    'GenomeData': GenomeData,
    'GenomeDataList': GenomeDataList
    }

original_conversion = robjects.conversion.ri2py
def bsgenome_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = pyobj.rclass[0]
        try:
            cls = _bsgenome_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

robjects.conversion.ri2py = bsgenome_conversion




import unittest

import rpy2.robjects as robjects
import bioc.biobase as bb

ridentical = robjects.r["identical"]

class AnnotatedDataFrameTestCase(unittest.TestCase):

    def setUp(self):
        self.adf = robjects.r["new"]("AnnotatedDataFrame")

    def test_pData(self):
        adf = self.adf
        pdata = adf.pdata
        self.assertTrue(robjects.r["is.data.frame"](pdata)[0])
        adf.pdata = pdata
        rpdata = robjects.r["pData"](adf)
        self.assertTrue(adf.pdata.rsame(rpdata))

    def test_combine(self):
        adf = self.adf
        cadf = adf.combine(adf)
        self.assertFalse(cadf.pdata.rsame(adf))
        self.assertTrue(ridentical(cadf, adf)[0])

    def test_featurenames(self):
        adf = self.adf
        featurenames = adf.featurenames
        self.assertEquals(len(featurenames), 
                          robjects.r["nrow"](adf.pdata)[0]) 

    def test_new(self):
        adf = bb.AnnotatedDataFrame.new()
        self.assertTrue(False) # test not properly implemented (yet)
    
class ExpressionSetTestCase(unittest.TestCase):

    def setUp(self):
        self.eset = robjects.r["new"]("ExpressionSet")

    def test_exprs(self):
        eset = self.eset
        exprs = eset.exprs
        self.assertTrue(robjects.r["is.matrix"](exprs)[0])
        eset.exprs = exprs
        rexprs = robjects.r["exprs"](eset)
        self.assertTrue(eset.exprs.rsame(rexprs))

    def test_new(self):
        
        eset = bb.ExpressionSet.new()

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(BiobaseTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ExpressionSetTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()




def testacs(instance, python_name):
    acs = getattr(instance.obj, python_name)
    if callable(acs):
        res = acs()
    else:
        res = acs

class RS4Test_Type(type):
    def __new__(mcs, name, bases, cls_dict):
        
        targetclass = cls_dict['__targetclass__']
        accessors = targetclass.__accessors__
        for rname, where, \
                python_name, as_property, \
                docstring in accessors:

            if python_name is None:
                python_name = rname
                
            cls_dict['test_' + python_name] = \
                             lambda self: testacs(self,
                                                  python_name)
            
        return type.__new__(mcs, name, bases, cls_dict)



#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the Biobase library in Bioconductor

Copyright 2009-2010 - Laurent Gautier

"""

import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages
import rpy2.rlike.container as rlc

getmethod = robjects.baseenv.get("getMethod")

__rname__ = 'Biobase'
robjects.packages.quiet_require(__rname__)

biobase_env = robjects.baseenv['as.environment']('package:Biobase')
__rpackage__ = robjects.packages.SignatureTranslatedPackage(biobase_env, __rname__)

new = robjects.conversion.ri2py(robjects.methods.methods_env['new'])

StrVector = robjects.StrVector


def _setExtractDelegators(self):
    self.rx = robjects.vectors.ExtractDelegator(self)
    self.rx2 = robjects.vectors.DoubleExtractDelegator(self)        


class Versioned(rpy2.robjects.methods.RS4):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('classVersion', 'Biobase', '__version__', True,
                      'maps Biobase::classVersion'),
                     )

class VersionedBiobase(Versioned):
    pass

class CharacterOrMIAME(rpy2.robjects.methods.RS4):
    """ Abstract class """
    pass

# class AssayData(rpy2.robjects.methods.RS4, rpy2.robjects.Environment):
#     """ Abstract class. That class in a ClassUnionRepresentation
#     in R, that a is way to create a parent class for existing classes.
#     This is currently not modelled in Python. """
#     pass

class AnnotatedDataFrame(Versioned):
    ''' An annotated data.frame as defined in the R package Biobase'''

    _combine = getmethod("combine", 
                         signature = StrVector(["AnnotatedDataFrame", 
                                                "AnnotatedDataFrame"]),
                         where="package:Biobase")
    _get_pData = getmethod("pData", 
                           signature = "AnnotatedDataFrame",
                           where="package:Biobase")
    _set_pData = getmethod("pData<-", 
                           signature = StrVector(["AnnotatedDataFrame",
                                                  "data.frame"]),
                           where="package:Biobase")
    _get_featureNames = getmethod("featureNames", 
                                  signature = "AnnotatedDataFrame",
                                  where="package:Biobase")
    _set_featureNames = getmethod("featureNames<-", 
                                  signature = StrVector(["AnnotatedDataFrame", ]),
                                  where="package:Biobase")


    @classmethod
    def new(cls,
            data = robjects.DataFrame(rlc.OrdDict()),
            varmetadata = robjects.DataFrame(rlc.OrdDict()),
            dimlabels = robjects.StrVector(("rowNames", "columnNames"))
            ):

        res = new("AnnotatedDataFrame", data = data,
                  varMetadata = varmetadata,
                  dimLabels = dimlabels)
        _setExtractDelegators(res)
        return res

    def combine(self, annotated_dataframe):
        """ Combine the instance with an other data.frame """
        res = self._combine(self, annotated_dataframe)
        return res

    def get_pdata(self):
        res = self._get_pData(self)
        return res

    def set_pdata(self, value):
        res = self._set_pData(self, value)
        self.__sexp__ = res.__sexp__

    pdata = property(get_pdata, set_pdata,
                     None,
                     "Property for both R's 'pData' and 'pData<-'")

    def get_featurenames(self):
        res = self._get_featureNames(self)
        return res

    def set_featurenames(self, value):
        res = self._set_featureNames(self, value)
        self.__sexp__ = res.__sexp__

    featurenames = property(get_featurenames, 
                            set_featurenames,
                            None,
                            "Property for both R's 'featureNames' and 'featureNames<-'")


class ESet(VersionedBiobase):
    ''' An eSet as defined in the R package Biobase.
        This class in defined as "virtual" in the R/S4 scheme,
        which can translates as "abstract" in more common OO terminologies. '''

    _assayData = getmethod("assayData", 
                           signature = StrVector(["eSet", ]), 
                           where="package:Biobase")
    _phenoData_get = getmethod("phenoData", 
                               signature = StrVector(["eSet", ]), 
                               where="package:Biobase")
    _phenoData_set = getmethod("phenoData<-", 
                               signature = StrVector(["eSet", 
                                                      "AnnotatedDataFrame"]), 
                               where="package:Biobase")
    _featureData_get = getmethod("featureData", 
                                 signature = StrVector(["eSet", ]), 
                                 where="package:Biobase")
    _featureData_set = getmethod("featureData<-", 
                                 signature = StrVector(["eSet", 
                                                        "AnnotatedDataFrame"]), 
                                 where="package:Biobase")
    _experimentData = getmethod("experimentData", 
                                signature = StrVector(["eSet", ]), 
                                where="package:Biobase")
    _annotation = getmethod("annotation", 
                            signature = StrVector(["eSet", ]), 
                            where="package:Biobase")

    def assaydata(self):
	res = self._assayData(self)
        return res

    def get_phenodata(self):
        res = self._phenoData_get(self)
        return res
    def set_phenodata(self, value):
        sexp = self._phenoData_set(self, value)
        self.__sexp__ = sexp
    property(get_phenodata, set_phenodata,
             None,
             "Property for both R's 'phenoData' and 'phenoData<-'")             
    
    def get_featuredata(self):
	res = self._featureData(self)
        return res
    def set_featuredata(self, value):
	sexp = self._featureData_set(self, value)
        self.__sexp__ = sexp
    property(get_featuredata, set_featuredata,
             None,
             "Property for both R's 'featureData' and 'featureData<-'")
        
    def experimentdata(self):
	res = self._experimentData(self)
        return res

    def annotation(self):
	res = self._annotation(self)
        return res

class ExpressionSet(ESet):

    _get_exprs = getmethod("exprs", 
                           signature = StrVector(["ExpressionSet", ]), 
                           where="package:Biobase")
    _set_exprs = getmethod("exprs<-", 
                           signature = StrVector(["ExpressionSet", "matrix"]), 
                           where="package:Biobase")
    _esApply = getmethod("esApply", 
                         signature = StrVector(["ExpressionSet", ]), 
                         where="package:Biobase")

    def __init__(self, *args, **kwargs):
        super(ExpressionSet, self).__init__(*args, **kwargs)
        self.rx = None
        self.rx2 = None
        
    @classmethod
    def new(cls,
            phenodata = new("AnnotatedDataFrame"),
            featuredata = new("AnnotatedDataFrame"),
            experimentdata = new("MIAME"),
            annotation = robjects.StrVector([]),
            exprs = new("matrix")):

        res = new("ExpressionSet", phenoData = phenodata,
                  featureData = featuredata,
                  experimentData = experimentdata,
                  annotation = annotation,
                  exprs = exprs)
        _setExtractDelegators(res)
        return res
            
    def get_exprs(self):
	res = self._get_exprs(self)
        return res 

    def set_exprs(self, value):
        res = self._set_exprs(self, value)
        return res

    exprs = property(get_exprs, set_exprs,
                     None,
                     "Property for both R's 'exprs' and 'exprs<-'")
    
    def esapply(self, margin, fun, *args):
        ''' "apply" a function "fun" along the "margin". '''
        #FIXME: kwargs not (yet) handled
        res = self._esapply(self, margin, fun, *args)
        return res

class SnpSet(ESet):
    pass

class NChannelSet(ESet):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('channelNames', 'Biobase', 'channelnames', 
                      True, 'maps Biobase::channelNames'),
                     )

    __channel = getmethod("channel", 
                          signature = StrVector(("NChannelSet", 
                                                 "character")),
                          where = "Biobase")
    
    __selectchannels = getmethod("selectChannels", 
                                 signature = StrVector(("NChannelSet", 
                                                        "character")),
                                 where = "Biobase")
    

    def channel(self, name, **kwargs):
        res = self.__channel(name, **kwargs)
        res = conversion.ri2py(res)
        return res

    def selectchannels(self, names, **kwargs):
        res = self.__selectchannels(names, **kwargs)
        res = conversion.ri2py(res)
        return res

class MultiSet(ESet):
    pass

class MIAME(CharacterOrMIAME):
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('abstract', 'Biobase', 'abstract', True,
                      'maps Biobase::abstract'),
                     ('expinfo', 'Biobase', 'expinfo', True,
                      'maps Biobase::expinfo'),
                     ('hybridizations', 'Biobase', 'hybridizations', True,
                      'maps Biobase::hybridizations'),
                     ('normControls', 'Biobase', 'normcontrols', True,
                      'maps Biobase::normControls'),
                     ('notes', 'Biobase', 'notes', True,
                      'maps Biobase::notes'),
                     ('otherInfo', 'Biobase', 'otherinfo', True,
                      'maps Biobase::otherInfo'),
                     ('preproc', 'Biobase', 'preproc', True,
                      'maps Biobase::preproc'),
                     ('samples', 'Biobase', 'samples', True,
                      'maps Biobase::samples'),
                     )


_biobase_dict = {
    'AnnotatedDataFrame': AnnotatedDataFrame,
    #'AssayData': AssayData,
    'chararacterORMIAME': CharacterOrMIAME,
    'eSet': ESet,
    'ExpressionSet': ExpressionSet,
    'MIAME': MIAME,
    'SnpSet': SnpSet,
    'Versioned': Versioned,
    'VersionedBiobase': VersionedBiobase
    }

original_conversion = robjects.conversion.ri2py
def biobase_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _biobase_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

robjects.conversion.ri2py = biobase_conversion






#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the IRanges library in Bioconductor

Copyright 2009 - Laurent Gautier

"""



import rpy2.robjects as robjects
import rpy2.robjects.packages
conversion = robjects.conversion

getmethod = robjects.baseenv.get("getMethod")

__rname__ = 'IRanges'
rpy2.robjects.packages.quiet_require(__rname__)

iranges_env = robjects.baseenv['as.environment']('package:IRanges')
__rpackage__ = robjects.packages.SignatureTranslatedPackage(iranges_env, __rname__)

StrVector = robjects.StrVector

class Sequence(robjects.methods.RS4):
    """ Abstract class """

class Rle(Sequence):
    _length = getmethod("length", 
                        signature = StrVector(["Rle", ]), 
                        where="package:IRanges")
    def __len__(self):
        res = self._length(self)
        return res[0]



class AtomicList(Sequence):
    pass


class LogicalList(AtomicList):
    pass

class IntegerList(AtomicList):
    pass

class NumericList(AtomicList):
    pass

class ComplexList(AtomicList):
    pass

class CharacterList(AtomicList):
    pass

class RawList(AtomicList):
    pass


class CompressedAtomicList(AtomicList):
    pass

class SimpleAtomicList(AtomicList):
    pass


class Ranges(robjects.RS4):
    _length = getmethod("length", 
                        signature = StrVector(["Ranges", ]), 
                        where="package:IRanges")
    def __len__(self):
        res = self._length(self)
        return res[0]

acs = (
    ('start', None, True, None),
    ('width', None, True, None),
    ('end', None, True, None),
    ('mid', None, False, None),
    ('isEmpty', 'isempty', False, None),
    ('isDisjoint', 'isdisjoint', False, None),
    ('isNormal', 'isnormal', False, None),
    ('whichFirstNotNormal', 'whichfirst_notnormal', False, None),
    ('as.matrix', 'as_matrix', False, None),
    ('as.integer', 'as_integer', False, None),
    ('as.data.frame', 'as_DataFrame', False, None)
    )
robjects.methods.set_accessors(Ranges,
                               "Ranges", "IRanges",
                               acs)
del(acs)

class XRanges(Ranges):
    pass

class IntervalTree(XRanges):
    _length = getmethod("length", 
                        signature = StrVector(["IntervalTree", ]), 
                        where="package:IRanges")
    def __len__(self):
        res = self._length(self)
        return res[0]



class IRanges(Ranges):
    pass

class NormalIRanges(IRanges):
    pass
acs = (
    ('max', None, True, None),
    ('min', None, True, None)
    )
robjects.methods.set_accessors(NormalIRanges,
                               "NormalIRanges", "IRanges",
                               acs)
del(acs)


class SimpleList(Sequence):
    _length = getmethod("length", 
                        signature = StrVector(["SimpleList", ]),
                        where="package:IRanges")
    def __len__(self):
        res = self._length(self)
        return res[0]

    _extract = getmethod("[",
                         signature = StrVector(["SimpleList", 
                                                "ANY", "ANY"]),
                         where="package:IRanges")
    def __getitem__(self, i):
        res = self._extract(self, i)
        return res

class AnnotatedList(robjects.RS4):
    pass

class DataFrame(SimpleList):
    _extract = getmethod("[[",
                         signature = StrVector(["DataFrame", ]),
                         where="package:IRanges")

    def __getitem__(self, i):
        res = self._extract(self, i)
        return res


class XDataFrameList(AnnotatedList):
    pass

class RangesList(AnnotatedList):
    pass

class RangesDataList(AnnotatedList):
    pass

class IRangesList(RangesList):
    pass

class Views(IRanges):
    _extract = getmethod("[[",
                         signature = StrVector(["Views", ]),
                         where="package:IRanges")
    def __getitem__(self, i):
        res = self._extract(self, i)
        return res

    _subject = getmethod("subject", 
                         signature = StrVector(["Views", ]), 
                         where="package:IRanges")

    def get_subject(self):
        """ Get the 'subject' """
        res = self._subject(self)
        return res
    subject = property(get_subject, None)

class RleViews(Views):
    pass

class MaskCollection(robjects.RS4):
    _length = getmethod("length", 
                        signature = "MaskCollection",
                        where = "package:IRanges")
    _mask_constructor = iranges_env['Mask']
    _width = getmethod("width",
                       signature = "MaskCollection",
                       where = "package:IRanges")

    @classmethod
    def new(cls, mask_width):
        res = cls(cls._mask_constructor(mask_width))
        return cls
        
    def __len__(self):
        res = self._length(self)[0]
        return res

    def get_width(self):
        res = self._length(self)
        return res
        
class RangedData(robjects.RS4):
    _length = getmethod("length", 
                        signature = "RangedData",
                        where = "package:IRanges")
    _names = getmethod("names", 
                        signature = "RangedData",
                        where = "package:IRanges")
    _extract = getmethod("[[",
                         signature = StrVector(["RangedData", ]),
                         where="package:IRanges")
    def __len__(self):
        res = self._length(self)[0]
        return res

    def get_names(self):
        res = self._names(self)
        return res
    names = property(get_names, None)

    def __getitem__(self, i):
        res = self._extract(self, i)
        return res


acs = (('start', None, True, None),
       ('width', None, True, None),
       ('end', None, True, None),
       )
robjects.methods.set_accessors(RangedData,
                               "RangedData", "IRanges",
                               acs)
del(acs)


_iranges_dict = {
    'Sequence': Sequence,
    'Rle': Rle,
    'AtomicList': AtomicList,
    'LogicalList': LogicalList,
    'IntegerList': IntegerList,
    'NumericList': NumericList,
    'CharacterList': CharacterList,
    'ComplexList': ComplexList,
    'RawList': RawList,
    'RangedData': RangedData,
    'Ranges': Ranges,
    'IRanges': IRanges,
    'XRanges': XRanges,
    'IntervalTree': IntervalTree,
    'NormalIRanges': NormalIRanges,
    'Views': Views,
    'RangesList': RangesList,
    'RangesDataList': RangesDataList,
    'IRangesList': IRangesList,
    'RleViews': RleViews,
    'DataFrame': DataFrame,
    'XDataFrameList': XDataFrameList
    }

original_conversion = conversion.ri2py
def iranges_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _iranges_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

conversion.ri2py = iranges_conversion




from distutils.core import setup


version = __import__('bioc').__version__

packages = ('bioc', )

setup(
    name = 'rpy2-bioconductor-extensions',
    version = version,
    description = "Bioconductor-specific extensions for rpy2",
    long_description = """rpy2.bioconductor-extensions proposes extensions to rpy2 for bioconductor/bioinformatics users""", 
    author = 'Laurent Gautier',
    author_email = 'lgautier@gmail.com',
    url = 'no url yet',
    license = 'GPLv3',
    platforms = ['any'],
    package_dir = {'bioc': 'bioc'},
    packages = packages,
    classifiers = ['Development Status :: 1 - Planning',
                   'Programming Language :: Python',
                   'License :: OSI Approved :: GNU Affero General Public License v3',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Science/Research',
                   'Operating System :: OS Independent'],
)





#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the GSEABase library in Bioconductor

Copyright 2009 - Laurent Gautier

"""

__rname__ = 'GSEABase'

import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages

rinterface = robjects.rinterface
conversion = robjects.conversion
getmethod = robjects.baseenv.get("getMethod")

import bioc.biobase, bioc.annotationdbi
rpy2.robjects.packages.quiet_require(__rname__)

package_env = robjects.baseenv['as.environment']('package:%s' %__rname__)
__rpackage__ = robjects.packages.SignatureTranslatedPackage(package_env, 
                                                            __rname__)

StrVector = robjects.StrVector

class GeneSet(rpy2.robjects.methods.RS4):
    __rname__ = 'GeneSet'
    __metaclass__ = rpy2.robjects.methods.RS4_Type
    
    __accessors__ = (('collectionType', 'GSEABase', 'collectiontype', True, None),
                     ('contributor', 'GSEABase', 'contributor', True, None),
                     ('creationDate', 'GSEABase', 'creationdate', True, None),
                     ('description', 'GSEABase', 'description', True, None),
                     ('geneIds', 'GSEABase', 'geneids', True, None),
                     ('longDescription', 'GSEABase', 'longdescription', True, None),
                     ('organism', 'GSEABase', 'organism', True, None),
                     ('pubMedIds', 'GSEABase', 'pubmedids', True, None),
                     ('setName', 'GSEABase', 'name', True, 'maps GSEABase::setName'),
                     ('geneIdType', 'GSEABase', 'geneidtype', True, 'maps GSEABase::geneIdType'),
                     ('urls', 'GSEABase', 'urls', True, None),
                     )

    __extract_num = getmethod("[",
                              signature = StrVector(["GeneSet", 
                                                     "numeric", "ANY", "ANY"]))

    __or = getmethod("|",
                     signature = StrVector(["GeneSet", "GeneSet"]))
    __and = getmethod("&",
                      signature = StrVector(["GeneSet", "GeneSet"]))
    __setdiff = getmethod("setdiff",
                          signature = StrVector(["GeneSet", "GeneSet"]))
                              #where="package:GSEABase") #likely bug in R's "methods" 
    #FIXME: duck-punching for the ExpressionSet class as the bioconductor
    # package defines a new "[" method using a GeneSet as an index
    def rx(self, i):
        res = self.__extract_num(self, i)
        res = conversion.ri2py(res)
        return res

    def union(self, gs):
        res = self.__or(self, gs)
        return conversion.rpi2py(res)

    def intersection(self, gs):
        res = self.__and(self, gs)
        return conversion.rpi2py(res)

    def difference(self, gs):
        res = self.__setdiff(self, gs)
        return conversion.rpi2py(res)


class GeneColorSet(GeneSet):
    __rname__ = 'GeneColorSet'
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('coloring', 'GSEABase', 'coloring', True, None),
                     ('geneColor', 'GSEABase', 'genecolor', True, None),
                     ('phenotypeColor', 'GSEABase', 'phenotypecolor', False, None),
                     ('phenotype', 'GSEABase', 'phenotype', False, None),
                     )

    __extract_num = getmethod("[",
                              signature = StrVector(['GeneColorSet', 
                                                     "numeric","ANY"]))
                              #where="package:GSEABase")
    __extract_str = getmethod("[",
                              signature = StrVector(['GeneColorSet', 
                                                     "character", "ANY"]))
                              #where="package:GSEABase")
    
    def rx(self, i):
        if inherits(i, str) or \
                (inherits(i, rinterface.SexpVector) and \
                     i.typeof == rinterface.STRSXP):
            res = self.__extract_str(self, i)
        else:
            res = self.__extract_num(self, i)
        res = conversion.ri2py(res)
        return res


class GeneSetCollection(rpy2.robjects.methods.RS4, rpy2.robjects.Vector):
    __rname__ = 'GeneSetCollection'
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('geneIds', 'GSEABase', 'geneids', True, None),
                     ('names', 'GSEABase', 'names', True, None)
                     )

_gseabase_dict = {
    'GeneSet': GeneSet,
    'GeneColorSet': GeneColorSet,
    'GeneSetCollection': GeneSetCollection
    }

original_conversion = robjects.conversion.ri2py
def gseabase_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _gseabase_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

robjects.conversion.ri2py = gseabase_conversion



#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A module to model the ShortRead library in Bioconductor

Copyright 2009 - Laurent Gautier

"""


import rpy2.robjects.methods
import rpy2.robjects as robjects
import rpy2.robjects.packages
conversion = robjects.conversion

getmethod = robjects.baseenv.get("getMethod")

import bioc.biobase

__rname__ = 'ShortRead'
rpy2.robjects.packages.quiet_require(__rname__)

shortread_env = robjects.baseenv['as.environment']('package:ShortRead')
__rpackage__ = robjects.packages.SignatureTranslatedPackage(shortread_env, __rname__)

StrVector = robjects.StrVector

class _ShortReadBase(robjects.methods.RS4):
    pass

class ShortRead(_ShortReadBase):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('length', 'ShortRead', '__len__', True, None),
                     ('width', 'ShortRead', None, True, None),
                     ('srorder', 'ShortRead', None, False, None),
                     ('srrank', 'ShortRead', None, False, None),
                     ('srsort', 'ShortRead', None, False, None),
                     ('srduplicated', 'ShortRead', None, False, None),
                     ('detail', 'ShortRead', None, False, None),
                     ('clean', 'ShortRead', None, False, None),
                     ('alphabetByCycle', 'ShortRead', 'alphabetbycycle', False, None)
                     )
    _tables = getmethod("tables", 
                        signature = StrVector(["ShortRead", ]), 
                        where="package:ShortRead")
    def tables(self, n = 50):
        res = self._tables(self, n = n)
        return res

    _narrow = getmethod("narrow", 
                        signature = StrVector(["ShortRead", ]), 
                        where="package:ShortRead")
    def narrow(self, start = None, end = None, width = None, use_names = True):
        res = self._narrow(self, start = start, end = end, 
                           width = width, use_names = use_names)
        return res

class ShortReadQ(ShortRead):
    pass

class AlignedRead(ShortReadQ):
    _coverage_coords = robjects.StrVector(('leftmost', 'fiveprime'))

    _coverage = getmethod("coverage", 
                          signature = StrVector(["AlignedRead", ]), 
                          where = "package:ShortRead")
    _extract = getmethod("[",
                         signature = StrVector(["AlignedRead", 
                                                "ANY", "missing"])
                         )

    def coverage(self, 
                 start = robjects.rinterface.NA_Logical,
                 end = robjects.rinterface.NA_Logical,
                 coords = _coverage_coords, 
                 extend = 0):
        res = self._coverage(self, start=start, end=end, coords=coords, extend=extend)
        return res

    def rx(self, i, drop = True):
        res = self._extract(self, i)
        res = conversion.ri2py(res)
        return res

class Intensity(_ShortReadBase):
    # __metaclass__ = rpy2.robjects.methods.RS4_Type

    # __accessors__ = (('readInfo', 'ShortRead', 'readinfo', True, None),
    #                  ('intensity', 'ShortRead', None, True, None),
    #                  ('measurementError', 'ShortRead', 'measurementerror', True, None),
    #                  ('dim', 'ShortRead', None, True, None),
    #                  )
    pass

class IntensityInfo(_ShortReadBase):
    pass
class IntensityMeasure(_ShortReadBase):
    pass
class ArrayIntensity(IntensityMeasure):
    pass

class QualityScore(_ShortReadBase):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (('length', 'ShortRead', '__len__', True, None),
                     ('width', 'ShortRead', None, True, None),
                     ('detail', 'ShortRead', None, False, None),
                     )

class NumericQuality(QualityScore):
    pass
class MatrixQuality(QualityScore):
    pass
class FastqQuality(QualityScore):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (
                     ('alphabetByCycle', 'ShortRead', 'alphabetbycycle', False, None),
                     ('alphabetFrequency', 'ShortRead', 'alphabetfrequency', False, None),
                     ('srorder', 'ShortRead', None, False, None),
                     ('srrank', 'ShortRead', None, False, None),
                     ('srduplicated', 'ShortRead', None, False, None),
                     )

class IntegerQuality(NumericQuality):
    pass

class SFastqQuality(FastqQuality):
    pass

class ExperimentPath(_ShortReadBase):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (
                     ('detail', 'ShortRead', None, False, None),
                     )

class _Solexa(_ShortReadBase):
    pass
class SolexaSet(_Solexa):
    pass
class _Roche(_ShortReadBase):
    pass
class RocheSet(_Roche):
    pass

class SolexaPath(ExperimentPath, _Solexa):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (
                     ('detail', 'ShortRead', None, False, None),
                     ('SolexaSet', 'ShortRead', 'solexaset', False, None),
                     )

    _readintensities =  getmethod("readIntensities", 
                                  signature = StrVector(["SolexaPath", ]), 
                                  where="package:ShortRead")

    _readprb =  getmethod("readPrb", 
                          signature = StrVector(["SolexaPath", ]), 
                          where="package:ShortRead")

    _readfastq =  getmethod("readFastq", 
                            signature = StrVector(["SolexaPath", ]), 
                            where="package:ShortRead")

    _readbasequality =  getmethod("readBaseQuality", 
                                  signature = StrVector(["SolexaPath", ]), 
                                  where="package:ShortRead")

    _readqseq =  getmethod("readQseq", 
                           signature = StrVector(["SolexaPath", ]), 
                           where="package:ShortRead")

    _readaligned =  getmethod("readAligned", 
                              signature = StrVector(["SolexaPath", ]), 
                              where="package:ShortRead")

    def readintensities(self, **kwargs):
        res = self._readintensities(self, **kwargs)
        return res

    def readprb(self, **kwargs):
        res = self._readprb(self, **kwargs)
        return res
    
    def readfastq(self, **kwargs):
        res = self._readfastq(self, **kwargs)
        return res
    
    def readqseq(self, **kwargs):
        res = self._readqseq(self, **kwargs)
        return res
    
    def readaligned(self, **kwargs):
        res = self._readaligned(self, **kwargs)
        return res

class RochePath(ExperimentPath, _Roche):
    __metaclass__ = rpy2.robjects.methods.RS4_Type

    __accessors__ = (
                     ('detail', 'ShortRead', None, False, None),
                     ('read454', 'ShortRead', None, False, None),
                     ('runNames', 'ShortRead', 'runnames', False, None),
                     ('RocheSet', 'ShortRead', 'rocheset', False, None),
                     )

    _readfasta =  getmethod("readFasta", 
                            signature = StrVector(["RochePath", ]), 
                            where="package:ShortRead")

    _readqual =  getmethod("readQual", 
                            signature = StrVector(["RochePath", ]), 
                            where="package:ShortRead")

    def readfasta(self, **kwargs):
        res = self._readfastq(self, **kwargs)
        return res
    def readqual(self, **kwargs):
        res = self._readqual(self, **kwargs)
        return res
    def readpath(self):
        res = self.do_slot("readPath")
        return res

class SRSet(_ShortReadBase):
    pass

class SolexaIntensity(Intensity):
    pass

class SolexaIntensityInfo(IntensityInfo):
    pass

class AlignedDataFrame(bioc.biobase.AnnotatedDataFrame):
    __append = getmethod("append",
                         signature = StrVector(["AlignedDataFrame", "AlignedDataFrame", "missing"]),
                         where = "package:ShortRead")

    def append(self, **kwargs):
        res = self.__append(self, **kwargs)
        return res


_shortread_dict = {
    '.ShortReadBase': _ShortReadBase,
    'ShortRead': ShortRead,
    'ShortReadQ': ShortReadQ,
    'AlignedRead': AlignedRead,
    'AlignedDataFrame': AlignedDataFrame,
    'Intensity': Intensity,
    'IntensityInfo': IntensityInfo,
    'IntensityMeasure': IntensityMeasure,
    'ArrayIntensity': ArrayIntensity,
    'QualityScore': QualityScore,
    'NumericQuality': NumericQuality,
    'MatrixQuality': MatrixQuality,
    'FastqQuality': FastqQuality,
    'IntegerQuality': IntegerQuality,
    'SFastqQuality': FastqQuality,
    'ExperimentPath': ExperimentPath,
    'SRSet': SRSet,
    'SolexaIntensity': SolexaIntensity,
    'SolexaIntensityInfo': SolexaIntensityInfo,
    'SolexaPath': SolexaPath,
    'SolexaSet': SolexaSet,
    'RochePath': RochePath,
    'RocheSet': RocheSet
    }

original_conversion = conversion.ri2py
def shortread_conversion(robj):

    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _shortread_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

conversion.ri2py = shortread_conversion




#    map bioconductor classes to Python classes 
#    Copyright (C) 2009-2010  Laurent Gautier
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The module reflects some of the content of the R/Bioconductor
package *GEOquery*. It defines Python-level classes
for the R/S4 classes, and gives otherwise access to
R-level commands the usual :mod:`rpy2:robjects` way.

In complement to this module, it is recommended to also use
'rpy2.robjects.packages.importr' to expose the full content of *GEOquery*

>>> import bioc.geoquery

Now all methods exposed by the R package are accessible from 'bio.geoquery.__rpackage__',
with Python classes defined in bioc.geoquery mapping the R classes.

"""

import rpy2.robjects as robjects
import rpy2.robjects.packages

conversion = robjects.conversion

getmethod = robjects.baseenv.get("getMethod")

__rname__ = 'GEOquery'
robjects.packages.quiet_require(__rname__)


geoquery_env = robjects.baseenv['as.environment']('package:GEOquery')
__rpackage__ = rpy2.robjects.packages.SignatureTranslatedPackage(geoquery_env, __rname__)

StrVector = robjects.StrVector

class GEOData(robjects.methods.RS4):
    """ R 'virtual' class (that is an abstract class) """
    __metaclass__ = robjects.methods.RS4_Type

    __accessors__ = (
        ('Table', 'GEOquery', 'table', True, "table"),
        ('dataTable', 'GEOquery', 'datatable', True, "data table"),
        ('Meta', 'GEOquery', 'meta', True, "meta-data (sample information)"),
        ('Columns', 'GEOquery', 'columns', True, "column descriptions"),
        ('Accession', 'GEOquery', 'accession', True, "GEO accession number")
        )

class GEODataTable(robjects.methods.RS4):
    __metaclass__ = robjects.methods.RS4_Type

    __accessors__ = (
        ('Table', 'GEOquery', 'table', True, 'table'),
        ('Columns', 'GEOquery', 'columns', True, 'column descriptions')
        )

class GPL(GEOData):
    """ GEO 'platforms' entity """
    pass

class GSM(GEOData):
    """ GEO 'samples' entity """
    pass

class GDS(GEOData):
    """ GEO 'datasets' entity """
    pass

_geoquery_dict = {
    'GEOData': GEOData,
    'GEODataTable': GEODataTable,
    'GSM': GSM,
    'GPL': GPL,
    'GDS': GDS
    }

original_conversion = conversion.ri2py
def geoquery_conversion(robj):
    pyobj = original_conversion(robj)
    if isinstance(pyobj, robjects.RS4):
        rclass = [x for x in pyobj.rclass][0]
        try:
            cls = _geoquery_dict[rclass]
            pyobj = cls(pyobj)
        except KeyError, ke:
            pass

    return pyobj

conversion.ri2py = geoquery_conversion
